<?php

namespace App\Controller;

use SplFileInfo;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Finder\Finder;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Process\Process;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Yaml\Yaml;

final class MainController extends AbstractController
{
    #[Route('/', 'app_root')]
    public function root(): Response
    {
        return $this->index('.index');
    }

    #[Route('/sync', 'app_sync')]
    public function sync(): Response
    {
        $gitDir = $this->getParameter('kernel.project_dir') . '/var/data';

        (new Process(['git', '-C', $gitDir, 'fetch']))->mustRun();
        (new Process(['git', '-C', $gitDir, 'reset', '--hard', 'FETCH_HEAD']))->mustRun();

        return new Response('Updated.');
    }

    #[Route('/{path}', 'app_index', ['path' => '.+'])]
    public function index(string $path): Response
    {
        $gitDir = $this->getParameter('kernel.project_dir') . '/var/data';


        // ---------------------------------------------- Get file contents ---

        $fullPath = sprintf(
            '%s/var/data/%s.md',
            $this->getParameter('kernel.project_dir'),
            $path,
        );

        $fs = new Filesystem();
        $contents = $fs->exists($fullPath) ? $fs->readFile($fullPath) : null;

        [
            'data' => $metadata,
            'text' => $text
        ] = $this->extractMetadata($contents);

        // -------------------------------------------------- Get data tree ---

        $finder = new Finder();
        $finder->files()->in($this->getParameter('kernel.project_dir') . '/var/data/');
        if (!$finder->hasResults()) throw $this->createNotFoundException();


        // Convert flat list into a nested tree
        $tree = [];
        foreach ($finder as $file) {
            $current = &$tree;

            foreach (explode('/', $file->getRelativePath()) as $part) {
                if ($part === '') break;

                if (!isset($current[$part])) {
                    $current[$part] = [];
                }
                $current = &$current[$part];
            }

            if (!isset($current[$part])) {
                $current[$file->getFilenameWithoutExtension()] = [];
            }
        }

        return $this->render('main/index.html.twig', [
            'path'     => $path,
            'contents' => $text,
            'metadata' => $metadata,
            'tree'     => $tree,
            'version'  => (new Process(['git', '-C', $gitDir, 'log', '-1', '--oneline']))->mustRun()->getOutput(),
        ]);
    }

    /**
     * Parses all YAML metadata blocks in a given string and returns the text
     * without such blocks.
     */
    private function extractMetadata(?string $text): array
    {
        if ($text === null) return ['data' => null, 'text' => null];

        $start = strpos($text, "---\n");
        $end = strpos($text, "...\n", $start) ?: strpos($text, "---\n", $start + 4);

        // No more moteadata, return as is. Recursion stop condition
        if ($start === false or $end === false) {
            return ['text' => $text, 'data' => []];
        }

        $yaml = Yaml::parse(substr($text, $start + 4, $end - $start - 4));
        $rest = substr($text, 0, $start) . substr($text, $end + 4);

        // Recurse until there's no more metadata blocks, overwriting as we go
        $nextIteration = $this->extractMetadata($rest);
        return [
            'data' => $nextIteration['data'] + $yaml,
            'text' => $nextIteration['text'],
        ];
    }
}
